@@ -29,6 +29,7 @@ from gi.repository import Grl, GLib, GObject
 # from gnomemusic.grilowrappers.grldleynawrapper import GrlDLeynaWrapper
 from gnomemusic.grilowrappers.grlsearchwrapper import GrlSearchWrapper
 from gnomemusic.grilowrappers.grltrackerwrapper import GrlTrackerWrapper
+from gnomemusic.trackerwrapper import TrackerState, TrackerWrapper
 
 
 class CoreGrilo(GObject.GObject):
@@ -45,7 +46,7 @@ class CoreGrilo(GObject.GObject):
     _theaudiodb_api_key = "195003"
 
     cover_sources = GObject.Property(type=bool, default=False)
-    tracker_available = GObject.Property(type=bool, default=False)
+    tracker_available = GObject.Property(type=int)
 
     def __repr__(self):
         return "<CoreGrilo>"
@@ -60,6 +61,14 @@ class CoreGrilo(GObject.GObject):
         self._thumbnail_sources_timeout = None
         self._wrappers = {}
 
+        self._tracker_wrapper = TrackerWrapper()
+        self._tracker_wrapper.bind_property(
+            "tracker-available", self, "tracker-available",
+            GObject.BindingFlags.SYNC_CREATE)
+
+        self._tracker_wrapper.connect(
+            "notify::tracker-available", self._on_tracker_available_changed)
+
         Grl.init(None)
 
         self._registry = Grl.Registry.get_default()
@@ -72,6 +81,14 @@ class CoreGrilo(GObject.GObject):
 
         self._registry.load_all_plugins(True)
 
+    def _on_tracker_available_changed(self, klass, value):
+        new_state = self._tracker_wrapper.props.tracker_available
+        # FIXME:No removal support yet.
+        if new_state == TrackerState.AVAILABLE:
+            # FIXME: Look for a better way to just activate the Tracker
+            # plugin.
+            self._registry.load_all_plugins(True)
+
     def _on_source_added(self, registry, source):
 
         def _trigger_art_update():
@@ -100,8 +117,10 @@ class CoreGrilo(GObject.GObject):
 
         new_wrapper = None
 
+        new_state = self._tracker_wrapper.props.tracker_available
         if (source.props.source_id == "grl-tracker-source"
-                and source.props.source_id not in self._wrappers.keys()):
+                and source.props.source_id not in self._wrappers.keys()
+                and new_state == TrackerState.AVAILABLE):
             new_wrapper = GrlTrackerWrapper(
                 source, self._coremodel, self._coreselection, self)
             self._wrappers[source.props.source_id] = new_wrapper
@@ -112,6 +131,7 @@ class CoreGrilo(GObject.GObject):
         #     self._wrappers.append(new_wrapper)
             print("wrapper", new_wrapper)
         elif (source.props.source_id not in self._search_wrappers.keys()
+                and source.props.source_id not in self._wrappers.keys()
                 and source.get_supported_media() & Grl.MediaType.AUDIO
                 and source.supported_operations() & Grl.SupportedOps.SEARCH):
             self._search_wrappers[source.props.source_id] = GrlSearchWrapper(
@@ -170,16 +190,20 @@ class CoreGrilo(GObject.GObject):
 
     def get_album_art_for_item(self, coresong, callback):
         # Tracker not (yet) loaded.
-        if "grl-tracker-source" not in self._wrappers:
+        if "grl-tracker-source" in self._wrappers:
             self._wrappers["grl-tracker-source"].get_album_art_for_item(
                 coresong, callback)
 
+    def get_artist_art(self, coreartist):
+        if "grl-tracker-source" in self._wrappers:
+            self._wrappers["grl-tracker-source"].get_artist_art(coreartist)
+
     def stage_playlist_deletion(self, playlist):
         """Prepares playlist deletion.
 
         :param Playlist playlist: playlist
         """
-        for wrapper in self._wrappers:
+        for wrapper in self._wrappers.values():
             if wrapper.source.props.source_id == "grl-tracker-source":
                 wrapper.stage_playlist_deletion(playlist)
                 break
@@ -190,7 +214,7 @@ class CoreGrilo(GObject.GObject):
         :param Playlist playlist: playlist
         :param bool deleted: indicates if the playlist has been deleted
         """
-        for wrapper in self._wrappers:
+        for wrapper in self._wrappers.values():
             if wrapper.source.props.source_id == "grl-tracker-source":
                 wrapper.finish_playlist_deletion(playlist, deleted)
                 break
@@ -201,7 +225,7 @@ class CoreGrilo(GObject.GObject):
         :param str playlist_title: playlist title
         :param callback: function to perform once, the playlist is created
         """
-        for wrapper in self._wrappers:
+        for wrapper in self._wrappers.values():
             if wrapper.source.props.source_id == "grl-tracker-source":
                 wrapper.create_playlist(playlist_title, callback)
                 break
