@@ -72,12 +72,14 @@ class CoreModel(GObject.GObject):
         "playlists-loaded": (GObject.SignalFlags.RUN_FIRST, None, ()),
     }
 
+    grilo = GObject.Property(type=CoreGrilo, default=None)
     songs_available = GObject.Property(type=bool, default=False)
 
     def __init__(self, coreselection):
         super().__init__()
 
         self._flatten_model = None
+        self._previous_playlist_model = None
         self._search_signal_id = None
         self._song_signal_id = None
 
@@ -106,10 +108,16 @@ class CoreModel(GObject.GObject):
             self._album_model)
         self._album_search_model.set_filter_func(lambda a: False)
 
+        self._album_search_filter = Gfm.FilterListModel.new(
+            self._album_search_model)
+
         self._artist_search_model = Dazzle.ListModelFilter.new(
             self._artist_model)
         self._artist_search_model.set_filter_func(lambda a: False)
 
+        self._artist_search_filter = Gfm.FilterListModel.new(
+            self._artist_search_model)
+
         self._playlists_model = Gio.ListStore.new(Playlist)
         self._playlists_model_filter = Dazzle.ListModelFilter.new(
             self._playlists_model)
@@ -118,7 +126,10 @@ class CoreModel(GObject.GObject):
         self._playlists_model_sort.set_sort_func(
             self._wrap_list_store_sort_func(self._playlists_sort))
 
-        self._grilo = CoreGrilo(self, self._coreselection)
+        self.props.grilo = CoreGrilo(self, self._coreselection)
+        # FIXME: Not all instances of internal _grilo use have been
+        # fixed.
+        self._grilo = self.props.grilo
 
         self._model.connect("items-changed", self._on_songs_items_changed)
 
@@ -182,7 +193,7 @@ class CoreModel(GObject.GObject):
         disc_model_sort.set_sort_func(
             self._wrap_list_store_sort_func(_disc_order_sort))
 
-        self._grilo.get_album_discs(media, disc_model)
+        self.props.grilo.get_album_discs(media, disc_model)
 
         return disc_model_sort
 
@@ -192,7 +203,7 @@ class CoreModel(GObject.GObject):
 
         albums_model_sort = Gfm.SortListModel.new(albums_model_filter)
 
-        self._grilo.get_artist_albums(media, albums_model_filter)
+        self.props.grilo.get_artist_albums(media, albums_model_filter)
 
         def _album_sort(album_a, album_b):
             return album_a.props.year > album_b.props.year
@@ -202,7 +213,14 @@ class CoreModel(GObject.GObject):
 
         return albums_model_sort
 
-    def set_playlist_model(self, playlist_type, model):
+    def set_player_model(self, playlist_type, model):
+        if model is self._previous_playlist_model:
+            for song in self._playlist_model:
+                if song.props.state == SongWidget.State.PLAYING:
+                    song.props.state = SongWidget.State.PLAYED
+
+            self.emit("playlist-loaded")
+            return
 
         def _on_items_changed(model, position, removed, added):
             if removed > 0:
@@ -214,22 +232,22 @@ class CoreModel(GObject.GObject):
                     coresong = model[position + i]
                     song = CoreSong(
                         coresong.props.media, self._coreselection,
-                        self._grilo)
+                        self.props.grilo)
 
                     self._playlist_model.insert(position + i, song)
 
                     song.bind_property(
                         "state", coresong, "state",
                         GObject.BindingFlags.SYNC_CREATE)
-                    song.bind_property(
-                        "validation", coresong, "validation",
-                        GObject.BindingFlags.SYNC_CREATE)
+                    coresong.bind_property(
+                        "validation", song, "validation",
+                        GObject.BindingFlags.BIDIRECTIONAL
+                        | GObject.BindingFlags.SYNC_CREATE)
 
         with model.freeze_notify():
+            self._playlist_model.remove_all()
 
             if playlist_type == PlayerPlaylist.Type.ALBUM:
-
-                self._playlist_model.remove_all()
                 proxy_model = Gio.ListStore.new(Gio.ListModel)
 
                 for disc in model:
@@ -242,19 +260,19 @@ class CoreModel(GObject.GObject):
                 for model_song in self._flatten_model:
                     song = CoreSong(
                         model_song.props.media, self._coreselection,
-                        self._grilo)
+                        self.props.grilo)
 
                     self._playlist_model.append(song)
                     song.bind_property(
                         "state", model_song, "state",
                         GObject.BindingFlags.SYNC_CREATE)
-                    song.bind_property(
-                        "validation", model_song, "validation",
-                        GObject.BindingFlags.SYNC_CREATE)
+                    model_song.bind_property(
+                        "validation", song, "validation",
+                        GObject.BindingFlags.BIDIRECTIONAL
+                        | GObject.BindingFlags.SYNC_CREATE)
 
                 self.emit("playlist-loaded")
             elif playlist_type == PlayerPlaylist.Type.ARTIST:
-                self._playlist_model.remove_all()
                 proxy_model = Gio.ListStore.new(Gio.ListModel)
 
                 for artist_album in model:
@@ -268,15 +286,16 @@ class CoreModel(GObject.GObject):
                 for model_song in self._flatten_model:
                     song = CoreSong(
                         model_song.props.media, self._coreselection,
-                        self._grilo)
+                        self.props.grilo)
 
                     self._playlist_model.append(song)
                     song.bind_property(
                         "state", model_song, "state",
                         GObject.BindingFlags.SYNC_CREATE)
-                    song.bind_property(
-                        "validation", model_song, "validation",
-                        GObject.BindingFlags.SYNC_CREATE)
+                    model_song.bind_property(
+                        "validation", song, "validation",
+                        GObject.BindingFlags.BIDIRECTIONAL
+                        | GObject.BindingFlags.SYNC_CREATE)
 
                 self.emit("playlist-loaded")
             elif playlist_type == PlayerPlaylist.Type.SONGS:
@@ -284,8 +303,6 @@ class CoreModel(GObject.GObject):
                     self._songliststore.props.model.disconnect(
                         self._song_signal_id)
 
-                self._playlist_model.remove_all()
-
                 for song in self._songliststore.props.model:
                     self._playlist_model.append(song)
 
@@ -301,8 +318,6 @@ class CoreModel(GObject.GObject):
                     self._song_search_flatten.disconnect(
                         self._search_signal_id)
 
-                self._playlist_model.remove_all()
-
                 for song in self._song_search_flatten:
                     self._playlist_model.append(song)
 
@@ -311,36 +326,31 @@ class CoreModel(GObject.GObject):
 
                 self.emit("playlist-loaded")
             elif playlist_type == PlayerPlaylist.Type.PLAYLIST:
-                # if self._search_signal_id:
-                #     self._song_search_model.disconnect(self._search_signal_id)
-
-                self._playlist_model.remove_all()
-
                 for model_song in model:
                     song = CoreSong(
                         model_song.props.media, self._coreselection,
-                        self._grilo)
+                        self.props.grilo)
 
                     self._playlist_model.append(song)
 
                     song.bind_property(
                         "state", model_song, "state",
                         GObject.BindingFlags.SYNC_CREATE)
-                    song.bind_property(
-                        "validation", model_song, "validation",
-                        GObject.BindingFlags.SYNC_CREATE)
-
-                # self._search_signal_id = self._song_search_model.connect(
-                #     "items-changed", _on_items_changed)
+                    model_song.bind_property(
+                        "validation", song, "validation",
+                        GObject.BindingFlags.BIDIRECTIONAL
+                        | GObject.BindingFlags.SYNC_CREATE)
 
                 self.emit("playlist-loaded")
 
+        self._previous_playlist_model = model
+
     def stage_playlist_deletion(self, playlist):
         """Prepares playlist deletion.
 
         :param Playlist playlist: playlist
         """
-        self._grilo.stage_playlist_deletion(playlist)
+        self.props.grilo.stage_playlist_deletion(playlist)
 
     def finish_playlist_deletion(self, playlist, deleted):
         """Finishes playlist deletion.
@@ -348,7 +358,7 @@ class CoreModel(GObject.GObject):
         :param Playlist playlist: playlist
         :param bool deleted: indicates if the playlist has been deleted
         """
-        self._grilo.finish_playlist_deletion(playlist, deleted)
+        self.props.grilo.finish_playlist_deletion(playlist, deleted)
 
     def create_playlist(self, playlist_title, callback):
         """Creates a new user playlist.
@@ -356,7 +366,7 @@ class CoreModel(GObject.GObject):
         :param str playlist_title: playlist title
         :param callback: function to perform once, the playlist is created
         """
-        self._grilo.create_playlist(playlist_title, callback)
+        self.props.grilo.create_playlist(playlist_title, callback)
 
     def activate_playlist(self, playlist):
         """Activates a playlist.
@@ -369,7 +379,7 @@ class CoreModel(GObject.GObject):
         self.emit("activate-playlist", playlist)
 
     def search(self, text):
-        self._grilo.search(text)
+        self.props.grilo.search(text)
 
     @GObject.Property(
         type=Gio.ListStore, default=None, flags=GObject.ParamFlags.READABLE)
@@ -427,12 +437,24 @@ class CoreModel(GObject.GObject):
     def albums_search(self):
         return self._album_search_model
 
+    @GObject.Property(
+        type=Gfm.FilterListModel, default=None,
+        flags=GObject.ParamFlags.READABLE)
+    def albums_search_filter(self):
+        return self._album_search_filter
+
     @GObject.Property(
         type=Dazzle.ListModelFilter, default=None,
         flags=GObject.ParamFlags.READABLE)
     def artists_search(self):
         return self._artist_search_model
 
+    @GObject.Property(
+        type=Gfm.FilterListModel, default=None,
+        flags=GObject.ParamFlags.READABLE)
+    def artists_search_filter(self):
+        return self._artist_search_filter
+
     @GObject.Property(
         type=Gtk.ListStore, default=None, flags=GObject.ParamFlags.READABLE)
     def songs_gtkliststore(self):
