@@ -87,6 +87,8 @@ class GrlTrackerWrapper(GObject.GObject):
         self._song_search_proxy = self._coremodel.props.songs_search_proxy
         self._album_search_model = self._coremodel.props.albums_search
         self._artist_search_model = self._coremodel.props.artists_search
+        self._batch_changed_media_ids = {}
+        self._content_changed_timeout = None
 
         self._song_search_tracker = Gfm.FilterListModel.new(self._model)
         self._song_search_tracker.set_filter_func(lambda a: False)
@@ -104,7 +106,7 @@ class GrlTrackerWrapper(GObject.GObject):
             source, coremodel, coreselection, grilo)
 
         self._source.notify_change_start()
-        self._source.connect("content-changed", self._on_content_changed)
+        self._source.connect("content-changed", self._batch_content_changed)
 
     @GObject.Property(
         type=Grl.Source, default=None, flags=GObject.ParamFlags.READABLE)
@@ -132,23 +134,42 @@ class GrlTrackerWrapper(GObject.GObject):
 
         return query
 
-    def _on_content_changed(self, source, medias, change_type, loc_unknown):
-        for media in medias:
-            if change_type == Grl.SourceChangeType.ADDED:
-                print("ADDED", media.get_id())
-                self._add_media(media)
-                self._check_album_change(media)
-                self._check_artist_change(media)
-            elif change_type == Grl.SourceChangeType.CHANGED:
-                print("CHANGED", media.get_id())
-                self._changed_media(media)
-            elif change_type == Grl.SourceChangeType.REMOVED:
-                print("REMOVED", media.get_id())
-                self._remove_media(media)
-                self._check_album_change(media)
-                self._check_artist_change(media)
+    def _batch_content_changed(self, source, medias, change_type, loc_unknown):
+        if medias == []:
+            return
 
-    def _check_album_change(self, media):
+        if change_type not in self._batch_changed_media_ids.keys():
+            self._batch_changed_media_ids[change_type] = []
+
+        [self._batch_changed_media_ids[change_type].append(media.get_id())
+            for media in medias]
+
+        if self._content_changed_timeout is None:
+            self._content_changed_timeout = GLib.timeout_add(
+                250, self._on_content_changed)
+
+    def _on_content_changed(self):
+        for change_type in self._batch_changed_media_ids.keys():
+            media_ids = self._batch_changed_media_ids[change_type]
+
+            # The Tracker indexed paths may differ from Music's paths.
+            # In that case Tracker will report it as 'changed', while
+            # it means 'added' to Music.
+            if (change_type == Grl.SourceChangeType.CHANGED
+                    or change_type == Grl.SourceChangeType.ADDED):
+                print("ADDED/CHANGED", media_ids)
+                self._changed_media(media_ids)
+            elif change_type == Grl.SourceChangeType.REMOVED:
+                print("REMOVED", media_ids)
+                self._remove_media(media_ids)
+
+        self._check_album_change()
+        self._check_artist_change()
+
+        self._batch_changed_media_ids = {}
+        self._content_changed_timeout = None
+
+    def _check_album_change(self):
         album_ids = {}
 
         query = """
@@ -203,7 +224,7 @@ class GrlTrackerWrapper(GObject.GObject):
         self._source.query(
             query, self.METADATA_KEYS, options, check_album_cb)
 
-    def _check_artist_change(self, media):
+    def _check_artist_change(self):
         artist_ids = {}
 
         query = """
@@ -252,22 +273,25 @@ class GrlTrackerWrapper(GObject.GObject):
         self._source.query(
             query, self.METADATA_KEYS, options, check_artist_cb)
 
-    def _remove_media(self, media):
-        try:
-            coresong = self._hash.pop(media.get_id())
-        except KeyError:
-            print("Removal KeyError")
-            return
+    def _remove_media(self, media_ids):
+        for media_id in media_ids:
+            try:
+                coresong = self._hash.pop(media_id)
+            except KeyError:
+                print("Removal KeyError")
+                return
 
-        for idx, coresong_model in enumerate(self._model):
-            if coresong_model is coresong:
-                print(
-                    "removing", coresong.props.media.get_id(),
-                    coresong.props.title)
-                self._model.remove(idx)
-                break
+            for idx, coresong_model in enumerate(self._model):
+                if coresong_model is coresong:
+                    print(
+                        "removing", coresong.props.media.get_id(),
+                        coresong.props.title)
+                    self._model.remove(idx)
+                    break
+
+    def _song_media_query(self, media_ids):
+        media_ids = str(media_ids)[1:-1]
 
-    def _song_media_query(self, media_id):
         query = """
         SELECT DISTINCT
             rdf:type(?song)
@@ -290,44 +314,17 @@ class GrlTrackerWrapper(GObject.GObject):
                 ?song nao:hasTag ?tag .
                 FILTER (?tag = nao:predefined-tag-favorite)
             }
-            FILTER ( tracker:id(?song) = %(media_id)s )
+            FILTER ( tracker:id(?song) in ( %(media_ids)s ) )
             %(location_filter)s
         }
         """.replace('\n', ' ').strip() % {
             'location_filter': self._location_filter(),
-            'media_id': media_id
+            'media_ids': media_ids
         }
 
         return query
 
-    def _add_media(self, media):
-
-        def _add_media(source, op_id, media, user_data, error):
-            if error:
-                print("ERROR", error)
-                return
-
-            if not media:
-                return
-
-            # FIXME: Figure out why we get double additions.
-            if media.get_id() in self._hash.keys():
-                print("ALREADY ADDED")
-                return
-
-            song = CoreSong(media, self._coreselection, self._grilo)
-            self._model.append(song)
-            self._hash[media.get_id()] = song
-
-            print("UPDATE ID", media.get_id(), media.get_title())
-
-        options = self._fast_options.copy()
-
-        self._source.query(
-            self._song_media_query(media.get_id()), self.METADATA_KEYS,
-            options, _add_media)
-
-    def _changed_media(self, media):
+    def _changed_media(self, media_ids):
 
         def _update_changed_media(source, op_id, media, user_data, error):
             if error:
@@ -337,12 +334,18 @@ class GrlTrackerWrapper(GObject.GObject):
             if not media:
                 return
 
-            self._hash[media.get_id()].update(media)
+            if media.get_id() not in self._hash:
+                print("Media not in hash", media.get_id())
+                song = CoreSong(media, self._coreselection, self._grilo)
+                self._model.append(song)
+                self._hash[media.get_id()] = song
+            else:
+                self._hash[media.get_id()].update(media)
 
         options = self._fast_options.copy()
 
         self._source.query(
-            self._song_media_query(media.get_id()), self.METADATA_KEYS,
+            self._song_media_query(media_ids), self.METADATA_KEYS,
             options, _update_changed_media)
 
     def _initial_songs_fill(self, source):
@@ -383,6 +386,7 @@ class GrlTrackerWrapper(GObject.GObject):
             }
             %(location_filter)s
         }
+        ORDER BY ?title
         """.replace('\n', ' ').strip() % {
             'location_filter': self._location_filter()
         }
@@ -422,7 +426,9 @@ class GrlTrackerWrapper(GObject.GObject):
             OPTIONAL { ?song nmm:composer/nmm:artistName ?composer . }
             OPTIONAL { ?album nmm:albumArtist/nmm:artistName ?album_artist . }
             %(location_filter)s
-        } GROUP BY ?album
+        }
+        GROUP BY ?album
+        ORDER BY ?title ?album_artist ?artist ?creation_date
         """.replace('\n', ' ').strip() % {
             'location_filter': self._location_filter()
         }
@@ -457,7 +463,9 @@ class GrlTrackerWrapper(GObject.GObject):
                     nmm:musicAlbum ?album;
                     nmm:performer ?artist .
             %(location_filter)s
-        } GROUP BY ?artist
+        }
+        GROUP BY ?artist
+        ORDER BY ?artist
         """.replace('\n', ' ').strip() % {
             'location_filter': self._location_filter()
         }
@@ -480,6 +488,7 @@ class GrlTrackerWrapper(GObject.GObject):
             rdf:type(?album)
             tracker:id(?album) AS ?id
             nie:title(?album) AS ?title
+            nie:contentCreated(?song) AS ?date
         WHERE {
             ?album a nmm:MusicAlbum .
             OPTIONAL { ?album  nmm:albumArtist ?album_artist . }
@@ -490,6 +499,8 @@ class GrlTrackerWrapper(GObject.GObject):
                      || tracker:id(?artist) = %(artist_id)s )
             %(location_filter)s
         }
+        GROUP BY ?album
+        ORDER BY ?date ?album
         """.replace('\n', ' ').strip() % {
             'artist_id': int(artist_id),
             'location_filter': self._location_filter()
@@ -538,6 +549,7 @@ class GrlTrackerWrapper(GObject.GObject):
             FILTER ( tracker:id(?album) = %(album_id)s )
             %(location_filter)s
         }
+        ORDER BY ?album_disc_number
         """.replace('\n', ' ').strip() % {
             'album_id': int(album_id),
             'location_filter': self._location_filter()
@@ -595,6 +607,7 @@ class GrlTrackerWrapper(GObject.GObject):
             )
             %(location_filter)s
         }
+        ORDER BY ?track_number
         """.replace('\n', ' ').strip() % {
             'album_id': album_id,
             'disc_nr': disc_nr,
@@ -605,10 +618,123 @@ class GrlTrackerWrapper(GObject.GObject):
         self._source.query(query, self.METADATA_KEYS, options, callback)
 
     def search(self, text):
+        # FIXME: Searches are limited to not bog down the UI with
+        # widget creation ({List,Flow}Box limitations). The limit is
+        # arbitrarily set to 50 and set in the Tracker query. It should
+        # be possible to set it through Grilo options instead. This
+        # does not work as expected and needs further investigation.
         term = Tracker.sparql_escape_string(
             GLib.utf8_normalize(
                 GLib.utf8_casefold(text, -1), -1, GLib.NormalizeMode.NFKD))
 
+        # Artist search
+
+        query = """
+        SELECT DISTINCT
+            rdf:type(?artist)
+            tracker:id(?artist) AS ?id
+        WHERE {
+            ?song a nmm:MusicPiece ;
+                    nmm:musicAlbum ?album ;
+                    nmm:performer ?artist .
+            BIND(tracker:normalize(
+                nmm:artistName(nmm:albumArtist(?album)), 'nfkd') AS ?match1) .
+            BIND(tracker:normalize(
+                nmm:artistName(nmm:performer(?song)), 'nfkd') AS ?match2) .
+            BIND(tracker:normalize(nmm:composer(?song), 'nfkd') AS ?match4) .
+            FILTER (
+                CONTAINS(tracker:case-fold(
+                    tracker:unaccent(?match1)), "%(name)s")
+                || CONTAINS(tracker:case-fold(?match1), "%(name)s")
+                || CONTAINS(tracker:case-fold(
+                    tracker:unaccent(?match2)), "%(name)s")
+                || CONTAINS(tracker:case-fold(?match2), "%(name)s")
+                || CONTAINS(tracker:case-fold(
+                    tracker:unaccent(?match4)), "%(name)s")
+                || CONTAINS(tracker:case-fold(?match4), "%(name)s")
+            )
+            %(location_filter)s
+        }
+        LIMIT 50
+        """.replace('\n', ' ').strip() % {
+            'location_filter': self._location_filter(),
+            'name': term
+        }
+
+        artist_filter_ids = []
+
+        def artist_filter(coreartist):
+            return coreartist.media.get_id() in artist_filter_ids
+
+        def artist_search_cb(source, op_id, media, data, error):
+            if error:
+                print("ERROR", error)
+                return
+
+            if not media:
+                self._artist_search_model.set_filter_func(artist_filter)
+                return
+
+            artist_filter_ids.append(media.get_id())
+
+        options = self._fast_options.copy()
+        self._source.query(
+            query, self.METADATA_KEYS, options, artist_search_cb)
+
+        # Album search
+
+        query = """
+        SELECT DISTINCT
+            rdf:type(nmm:musicAlbum(?song))
+            tracker:id(nmm:musicAlbum(?song)) AS ?id
+        WHERE {
+            ?song a nmm:MusicPiece .
+            BIND(tracker:normalize(
+                nie:title(nmm:musicAlbum(?song)), 'nfkd') AS ?match1) .
+            BIND(tracker:normalize(
+                nmm:artistName(nmm:performer(?song)), 'nfkd') AS ?match2) .
+            BIND(tracker:normalize(nmm:composer(?song), 'nfkd') AS ?match4) .
+            FILTER (
+                CONTAINS(tracker:case-fold(
+                    tracker:unaccent(?match1)), "%(name)s")
+                || CONTAINS(tracker:case-fold(?match1), "%(name)s")
+                || CONTAINS(tracker:case-fold(
+                    tracker:unaccent(?match2)), "%(name)s")
+                || CONTAINS(tracker:case-fold(?match2), "%(name)s")
+                || CONTAINS(tracker:case-fold(
+                    tracker:unaccent(?match4)), "%(name)s")
+                || CONTAINS(tracker:case-fold(?match4), "%(name)s")
+            )
+            %(location_filter)s
+        }
+        LIMIT 50
+        """.replace('\n', ' ').strip() % {
+            'location_filter': self._location_filter(),
+            'name': term
+        }
+
+        album_filter_ids = []
+
+        def album_filter(corealbum):
+            return corealbum.media.get_id() in album_filter_ids
+
+        def albums_search_cb(source, op_id, media, data, error):
+            if error:
+                print("ERROR", error)
+                return
+
+            if not media:
+                self._album_search_model.set_filter_func(album_filter)
+                return
+
+            album_filter_ids.append(media.get_id())
+
+        options = self._fast_options.copy()
+        self._source.query(
+            query, self.METADATA_KEYS, options, albums_search_cb)
+
+        # Song search
+
         query = """
         SELECT DISTINCT
             rdf:type(?song)
@@ -639,6 +765,7 @@ class GrlTrackerWrapper(GObject.GObject):
             )
             %(location_filter)s
         }
+        LIMIT 50
         """.replace('\n', ' ').strip() % {
             'location_filter': self._location_filter(),
             'name': term
@@ -664,118 +791,6 @@ class GrlTrackerWrapper(GObject.GObject):
 
         self._source.query(query, self.METADATA_KEYS, options, songs_search_cb)
 
-        # Album search
-
-        query = """
-        SELECT DISTINCT
-            rdf:type(nmm:musicAlbum(?song))
-            tracker:id(nmm:musicAlbum(?song)) AS ?id
-        WHERE {
-            ?song a nmm:MusicPiece .
-            BIND(tracker:normalize(
-                nie:title(nmm:musicAlbum(?song)), 'nfkd') AS ?match1) .
-            BIND(tracker:normalize(
-                nmm:artistName(nmm:performer(?song)), 'nfkd') AS ?match2) .
-            BIND(tracker:normalize(nie:title(?song), 'nfkd') AS ?match3) .
-            BIND(tracker:normalize(nmm:composer(?song), 'nfkd') AS ?match4) .
-            FILTER (
-                CONTAINS(tracker:case-fold(
-                    tracker:unaccent(?match1)), "%(name)s")
-                || CONTAINS(tracker:case-fold(?match1), "%(name)s")
-                || CONTAINS(tracker:case-fold(
-                    tracker:unaccent(?match2)), "%(name)s")
-                || CONTAINS(tracker:case-fold(?match2), "%(name)s")
-                || CONTAINS(tracker:case-fold(
-                    tracker:unaccent(?match3)), "%(name)s")
-                || CONTAINS(tracker:case-fold(?match3), "%(name)s")
-                || CONTAINS(tracker:case-fold(
-                    tracker:unaccent(?match4)), "%(name)s")
-                || CONTAINS(tracker:case-fold(?match4), "%(name)s")
-            )
-            %(location_filter)s
-        }
-        """.replace('\n', ' ').strip() % {
-            'location_filter': self._location_filter(),
-            'name': term
-        }
-
-        album_filter_ids = []
-
-        def album_filter(corealbum):
-            return corealbum.media.get_id() in album_filter_ids
-
-        def albums_search_cb(source, op_id, media, data, error):
-            if error:
-                print("ERROR", error)
-                return
-
-            if not media:
-                self._album_search_model.set_filter_func(album_filter)
-                return
-
-            album_filter_ids.append(media.get_id())
-
-        options = self._fast_options.copy()
-        self._source.query(
-            query, self.METADATA_KEYS, options, albums_search_cb)
-
-        # Artist search
-
-        query = """
-        SELECT DISTINCT
-            rdf:type(?artist)
-            tracker:id(?artist) AS ?id
-        WHERE {
-            ?song a nmm:MusicPiece ;
-                    nmm:musicAlbum ?album ;
-                    nmm:performer ?artist .
-            BIND(tracker:normalize(
-                nie:title(nmm:musicAlbum(?song)), 'nfkd') AS ?match1) .
-            BIND(tracker:normalize(
-                nmm:artistName(nmm:performer(?song)), 'nfkd') AS ?match2) .
-            BIND(tracker:normalize(nie:title(?song), 'nfkd') AS ?match3) .
-            BIND(tracker:normalize(nmm:composer(?song), 'nfkd') AS ?match4) .
-            FILTER (
-                CONTAINS(tracker:case-fold(
-                    tracker:unaccent(?match1)), "%(name)s")
-                || CONTAINS(tracker:case-fold(?match1), "%(name)s")
-                || CONTAINS(tracker:case-fold(
-                    tracker:unaccent(?match2)), "%(name)s")
-                || CONTAINS(tracker:case-fold(?match2), "%(name)s")
-                || CONTAINS(tracker:case-fold(
-                    tracker:unaccent(?match3)), "%(name)s")
-                || CONTAINS(tracker:case-fold(?match3), "%(name)s")
-                || CONTAINS(tracker:case-fold(
-                    tracker:unaccent(?match4)), "%(name)s")
-                || CONTAINS(tracker:case-fold(?match4), "%(name)s")
-            )
-            %(location_filter)s
-        }
-        """.replace('\n', ' ').strip() % {
-            'location_filter': self._location_filter(),
-            'name': term
-        }
-
-        artist_filter_ids = []
-
-        def artist_filter(coreartist):
-            return coreartist.media.get_id() in artist_filter_ids
-
-        def artist_search_cb(source, op_id, media, data, error):
-            if error:
-                print("ERROR", error)
-                return
-
-            if not media:
-                self._artist_search_model.set_filter_func(artist_filter)
-                return
-
-            artist_filter_ids.append(media.get_id())
-
-        options = self._fast_options.copy()
-        self._source.query(
-            query, self.METADATA_KEYS, options, artist_search_cb)
-
     def get_album_art_for_item(self, coresong, callback):
         """Placeholder until we got a better solution
         """
@@ -792,7 +807,7 @@ class GrlTrackerWrapper(GObject.GObject):
             | Grl.ResolutionFlags.IDLE_RELAY)
         full_options.set_count(1)
 
-        self.search_source.query(
+        self._source.query(
             query, self.METADATA_THUMBNAIL_KEYS, full_options, callback)
 
     def _get_album_for_album_id(self, album_id):
@@ -854,6 +869,25 @@ class GrlTrackerWrapper(GObject.GObject):
 
         return query
 
+    def get_artist_art(self, coreartist):
+        media = coreartist.props.media
+
+        def _resolve_cb(source, op_id, resolved_media, data, error):
+            if resolved_media.get_thumbnail() is None:
+                coreartist.props.thumbnail = ""
+                return
+
+            media.set_thumbnail(resolved_media.get_thumbnail())
+            coreartist.props.thumbnail = media.get_thumbnail()
+
+        full_options = Grl.OperationOptions()
+        full_options.set_resolution_flags(
+            Grl.ResolutionFlags.FULL | Grl.ResolutionFlags.IDLE_RELAY)
+
+        self._source.resolve(
+            media, [Grl.METADATA_KEY_THUMBNAIL], full_options, _resolve_cb,
+            None)
+
     def stage_playlist_deletion(self, playlist):
         """Prepares playlist deletion.
 
