@@ -1,4 +1,4 @@
-# Copyright (c) 2016 The GNOME Music Developers
+# Copyright 2019 The GNOME Music Developers
 #
 # GNOME Music is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,77 +25,76 @@
 from gettext import gettext as _
 from gi.repository import GObject, Gtk
 
-from gnomemusic import log
-from gnomemusic.views.baseview import BaseView
 from gnomemusic.widgets.headerbar import HeaderBar
 from gnomemusic.widgets.albumcover import AlbumCover
 from gnomemusic.widgets.albumwidget import AlbumWidget
 
 
-class AlbumsView(BaseView):
+@Gtk.Template(resource_path="/org/gnome/Music/ui/AlbumsView.ui")
+class AlbumsView(Gtk.Stack):
+    """Gridlike view of all albums
+
+    Album activation switches to AlbumWidget.
+    """
+
+    __gtype_name__ = "AlbumsView"
 
     search_mode_active = GObject.Property(type=bool, default=False)
+    selected_items_count = GObject.Property(type=int, default=0, minimum=0)
+    selection_mode = GObject.Property(type=bool, default=False)
+
+    _all_albums = Gtk.Template.Child()
+    _flowbox = Gtk.Template.Child()
 
     def __repr__(self):
         return '<AlbumsView>'
 
-    @log
-    def __init__(self, window, player):
-        self._window = window
-        super().__init__('albums', _("Albums"), window)
+    def __init__(self, application, player=None):
+        """Initialize AlbumsView
 
-        self.player = player
-        self._album_widget = AlbumWidget(player, self)
+        :param application: The Application object
+        """
+        super().__init__(transition_type=Gtk.StackTransitionType.CROSSFADE)
+
+        # FIXME: Make these properties.
+        self.name = "albums"
+        self.title = _("Albums")
+
+        self._window = application.props.window
+        self._headerbar = self._window._headerbar
+
+        model = self._window._app.props.coremodel.props.albums_sort
+        self._flowbox.bind_model(model, self._create_widget)
+        self._flowbox.connect("child-activated", self._on_child_activated)
+
+        self.connect("notify::selection-mode", self._on_selection_mode_changed)
+
+        self.bind_property(
+            'selection-mode', self._window, 'selection-mode',
+            GObject.BindingFlags.BIDIRECTIONAL)
+
+        self._album_widget = AlbumWidget(application.props.player, self)
         self._album_widget.bind_property(
             "selection-mode", self, "selection-mode",
             GObject.BindingFlags.BIDIRECTIONAL)
 
         self.add(self._album_widget)
-        self.albums_selected = []
-        self.all_items = []
-        self.items_selected = []
-        self.items_selected_callback = None
 
         self.connect(
             "notify::search-mode-active", self._on_search_mode_changed)
 
-    @log
+        self.show_all()
+
     def _on_selection_mode_changed(self, widget, data=None):
-        super()._on_selection_mode_changed(widget, data)
-
         if not self.props.selection_mode:
-            self._on_changes_pending()
+            self.unselect_all()
 
-    @log
     def _on_search_mode_changed(self, klass, param):
         if (not self.props.search_mode_active
                 and self._headerbar.props.stack.props.visible_child == self
                 and self.get_visible_child() == self._album_widget):
             self._set_album_headerbar(self._album_widget.props.album)
 
-    @log
-    def _setup_view(self):
-        self._view = Gtk.FlowBox(
-            homogeneous=True, hexpand=True, halign=Gtk.Align.FILL,
-            valign=Gtk.Align.START, selection_mode=Gtk.SelectionMode.NONE,
-            margin=18, row_spacing=12, column_spacing=6,
-            min_children_per_line=1, max_children_per_line=20, visible=True)
-
-        self._view.get_style_context().add_class('content-view')
-        self._view.connect('child-activated', self._on_child_activated)
-
-        scrolledwin = Gtk.ScrolledWindow()
-        scrolledwin.add(self._view)
-        scrolledwin.show()
-
-        self._box.add(scrolledwin)
-
-        self._model = self._window._app.props.coremodel.props.albums_sort
-        self._view.bind_model(self._model, self._create_widget)
-
-        self._view.show()
-
-    @log
     def _create_widget(self, corealbum):
         album_widget = AlbumCover(corealbum)
 
@@ -113,12 +112,10 @@ class AlbumsView(BaseView):
 
         return album_widget
 
-    @log
     def _back_button_clicked(self, widget, data=None):
         self._headerbar.state = HeaderBar.State.MAIN
-        self.set_visible_child(self._grid)
+        self.props.visible_child = self._all_albums
 
-    @log
     def _on_child_activated(self, widget, child, user_data=None):
         corealbum = child.props.corealbum
         if self.props.selection_mode:
@@ -130,25 +127,18 @@ class AlbumsView(BaseView):
         self._set_album_headerbar(corealbum)
         self.set_visible_child(self._album_widget)
 
-    @log
     def _set_album_headerbar(self, corealbum):
         self._headerbar.props.state = HeaderBar.State.CHILD
         self._headerbar.props.title = corealbum.props.title
         self._headerbar.props.subtitle = corealbum.props.artist
 
-    @log
-    def _populate(self, data=None):
-        # self._window.notifications_popup.push_loading()
-        self._init = True
-        self._view.show()
-
     def _toggle_all_selection(self, selected):
         """
         Selects or unselects all items without sending the notify::active
         signal for performance purposes.
         """
         with self._window._app.props.coreselection.freeze_notify():
-            for child in self._view.get_children():
+            for child in self._flowbox.get_children():
                 child.props.selected = selected
                 child.props.corealbum.props.selected = selected
 
