@@ -45,23 +45,24 @@ class ArtistsView(BaseView):
         return '<ArtistsView>'
 
     @log
-    def __init__(self, window, player):
+    def __init__(self, application, player):
         """Initialize
 
-        :param GtkWidget window: The main window
+        :param GtkApplication application: The application object
         :param player: The main player object
         """
         self._sidebar = Gtk.ListBox()
         sidebar_container = Gtk.ScrolledWindow()
         sidebar_container.add(self._sidebar)
 
-        super().__init__('artists', _("Artists"), window, sidebar_container)
+        super().__init__(
+            'artists', _("Artists"), application, sidebar_container)
 
-        self.player = player
+        self._application = application
         self._artists = {}
 
-        self._window = window
-        self._coremodel = window._app.props.coremodel
+        self._window = application.props.window
+        self._coremodel = application.props.coremodel
         self._model = self._coremodel.props.artists_sort
 
         self._model.connect_after(
@@ -134,18 +135,6 @@ class ArtistsView(BaseView):
             vhomogeneous=False)
         self._view_container.add(self._view)
 
-        empty_frame = Gtk.Frame(shadow_type=Gtk.ShadowType.NONE, hexpand=True)
-        empty_frame.show()
-        self._view.add_named(empty_frame, "empty-frame")
-
-    @log
-    def _on_changes_pending(self, data=None):
-        if (self._init
-                and not self.props.selection_mode):
-            self._artists.clear()
-            self._offset = 0
-            self._populate()
-
     @log
     def _on_artist_activated(self, sidebar, row, data=None):
         """Initializes new artist album widgets"""
@@ -162,40 +151,19 @@ class ArtistsView(BaseView):
             self._view.set_visible_child_name(coreartist.props.artist)
             return
 
-        if self._loading_id > 0:
-            self._artist_albums.disconnect(self._loading_id)
-            self._loading_id = 0
-
         self._artist_albums = ArtistAlbumsWidget(
-            coreartist, self.player, self._window, False)
-        self._loading_id = self._artist_albums.connect(
-            "ready", self._on_artist_albums_ready, coreartist.props.artist)
-        self._view.set_visible_child_name("empty-frame")
-        self._window.notifications_popup.push_loading()
-        return
-
-    def _on_artist_albums_ready(self, widget, artist):
+            coreartist, self._application, False)
         artist_albums_frame = Gtk.Frame(
             shadow_type=Gtk.ShadowType.NONE, hexpand=True)
         artist_albums_frame.add(self._artist_albums)
         artist_albums_frame.show()
 
-        self._view.add_named(artist_albums_frame, artist)
+        self._view.add_named(artist_albums_frame, coreartist.props.artist)
         scroll_vadjustment = self._view_container.props.vadjustment
         scroll_vadjustment.props.value = 0.
         self._view.set_visible_child(artist_albums_frame)
-        self._window.notifications_popup.pop_loading()
-        self._loaded_artists.append(artist)
 
-        self._artist_albums.disconnect(self._loading_id)
-        self._loading_id = 0
-        self._artist_albums = None
-        return
-
-    @log
-    def _populate(self, data=None):
-        """Populates the view"""
-        pass
+        self._loaded_artists.append(coreartist.props.artist)
 
     @log
     def _on_sidebar_clicked(self, gesture, n_press, x, y):
