@@ -32,6 +32,7 @@ from gi.repository.Dazzle import BoldingLabel  # noqa: F401
 from gnomemusic import log
 from gnomemusic import utils
 from gnomemusic.coresong import CoreSong
+from gnomemusic.utils import SongStateIcon
 from gnomemusic.widgets.starimage import StarImage  # noqa: F401
 
 
@@ -56,6 +57,7 @@ class SongWidget(Gtk.EventBox):
     }
 
     coresong = GObject.Property(type=CoreSong, default=None)
+    select_click = GObject.Property(type=bool, default=False)
     selected = GObject.Property(type=bool, default=False)
     show_duration = GObject.Property(type=bool, default=True)
     show_favorite = GObject.Property(type=bool, default=True)
@@ -96,7 +98,6 @@ class SongWidget(Gtk.EventBox):
         super().__init__()
 
         self.props.coresong = coresong
-        self._media = self.props.coresong.props.media
         self._selection_mode = False
         self._state = SongWidget.State.UNPLAYED
 
@@ -210,6 +211,12 @@ class SongWidget(Gtk.EventBox):
 
         self.emit("widget-moved", source_position)
 
+    @Gtk.Template.Callback()
+    def _on_select_button_toggled(self, widget):
+        # This property is used to ignore the second click event
+        # (one event in SongWidget and the other one in select_button).
+        self.props.select_click = not self.props.select_click
+
     @Gtk.Template.Callback()
     @log
     def _on_star_toggle(self, widget, event):
@@ -294,7 +301,7 @@ class SongWidget(Gtk.EventBox):
     def _on_validation_changed(self, coresong, sate):
         validation_status = coresong.props.validation
         if validation_status == CoreSong.Validation.FAILED:
-            self._play_icon.props.icon_name = "dialog-error-symbolic"
+            self._play_icon.props.icon_name = SongStateIcon.ERROR.value
             self._play_icon.set_visible(True)
         else:
-            self._play_icon.props.icon_name = "media-playback-start-symbolic"
+            self._play_icon.props.icon_name = SongStateIcon.PLAYING.value
